#!/bin/sh

cd `dirname $0`

COLOR_RED="\033[1;31m"
COLOR_GREEN="\033[1;32m"
COLOR_1="\033[31m"
COLOR_OFF="\033[m"

ARCHUNAME=`uname -m`
case $ARCHUNAME in
    x86_64)
	;;
    i686)
	;;
    i586)
	;;
    i486)
	;;
    i386)
	;;
    *)	
	echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] architecture not supported."
	exit 1
	;;
esac

ARCH=`getconf LONG_BIT`
case $ARCH in
    64)
	TELI_DIRNAME=$PWD/x64
	SO_PATH_FILE=telicamsdk_x64.conf
	;;
    32)
	TELI_DIRNAME=$PWD/x86
	SO_PATH_FILE=telicamsdk_x86.conf
	;;
    *)
	;;
esac

OPT_TELICAMSDK=/opt/TeliCamSDK
BIN_TELICAMSDK=/opt/TeliCamSDK/bin
HOME_TELICAMSDK=/home/TeliCamSDK
HOME_DIR_TELICAMSDK=$HOME/TeliCamSDK
HOME_DIR_TELICAMSDK_GENICAM=$HOME/TeliCamSDK/genicam
TELI_RULE_FILE=80-telicamsdk.rules
RULE_DIR=/etc/udev/rules.d
TELICAMVIEWER_LAUNCHER=/usr/share/applications/telicamlinuxviewer.desktop
ROOT_TELICAMSDK=/root/TeliCamSDK
ROOT_TELICAMSDK_GENICAM=/root/TeliCamSDK/genicam/cache
SO_PATH_DIR=/etc/ld.so.conf.d
PROFILE_DIR=/etc/profile.d
PROFILE_SH=telicamsdk.sh

ECHOOP=""
if [ -e /etc/redhat-release ] ; then
    GETDISTRIB=`cat /etc/redhat-release | awk 'NR == 1 {print $1}'`
else
    GETDISTRIB=`cat /etc/issue | awk 'NR == 1 {print $1}'`
fi

case $GETDISTRIB in
    Ubuntu)
	;;
    Debian)
	;;
    Fedora)
	ECHOOP="-e"
	;;
    CentOS)
	ECHOOP="-e"
	;;
    *)
	;;
esac

echo "
Welcome to TeliCamSDK"

echo $ECHOOP "
$COLOR_1=========================================================$COLOR_OFF

 Please read the license agreement carefully.

 $PWD/license.txt

$COLOR_1=========================================================$COLOR_OFF"
echo ""
read -p "Do you accept the terms in the license agreement?[y/n]:" LicenseAgreementYesNo
case $LicenseAgreementYesNo in
    [Yy]*)
	echo "You have accepted the license agreement."
	echo ""
	read -p "Do you want to start the setup?[y/n]: " SetupYesNo
	case $SetupYesNo in
	    [Yy]*)
		
		SUDO=""
		if [ `id -u` != "0" ] ; then
		    SUDO="sudo"
		fi
		
		echo ""
		echo "Setting up TeliCamSDK api."
		if [ -e "$OPT_TELICAMSDK" ] ; then
		    read -p "TeliCamSDK api exists. Do you want to setup the TeliCamSDK api?[y/n]:" ApiReinstallYesNo
		    case $ApiReinstallYesNo in
			[Yy]*)
			    $SUDO rm -rf $OPT_TELICAMSDK
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 2"
				exit 2
			    fi
			    
			    $SUDO cp -ar "$TELI_DIRNAME"$OPT_TELICAMSDK /opt/
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 3"
				exit 3
			    fi
			    
			    $SUDO cp -ar "$PWD"/documents $OPT_TELICAMSDK
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 4"
				exit 4
			    fi

			    $SUDO chmod 775 $BIN_TELICAMSDK/TeliCamLinuxViewer $BIN_TELICAMSDK/execute_TeliCamLinuxViewer.sh
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 5"
				exit 5
			    fi

			    $SUDO chown -R root:root $OPT_TELICAMSDK
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 6"
				exit 6
			    fi

			    if [ "$SUDO" ] ; then
				ROOT_TELICAMSDK_JUDGE=`$SUDO find /root/TeliCamSDK -name "genicam" 2>/dev/null | awk -F/ '{print $NF}'`
				if [ "$ROOT_TELICAMSDK_JUDGE" ] ; then
				    $SUDO rm -rf $ROOT_TELICAMSDK
				    if [ $? != 0 ] ; then
					echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 7"
					exit 7
				    fi
				fi

				$SUDO mkdir -p $ROOT_TELICAMSDK_GENICAM
				if [ $? != 0 ] ; then
				    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 8"
				    exit 8
				fi
			    fi

			    $SUDO rm -rf $SO_PATH_DIR/$SO_PATH_FILE
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 9"
				exit 9
			    fi
			    $SUDO cp -ar "$PWD"$SO_PATH_DIR/$SO_PATH_FILE $SO_PATH_DIR
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 10"
				exit 10
			    fi
			    $SUDO chmod 644 $SO_PATH_DIR/$SO_PATH_FILE
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 11"
				exit 11
			    fi
			    $SUDO chown root:root $SO_PATH_DIR/$SO_PATH_FILE
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 12"
				exit 12
			    fi
			    $SUDO ldconfig
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 13"
				exit 13
			    fi

			    $SUDO rm -rf $PROFILE_DIR/$PROFILE_SH
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 14"
				exit 14
			    fi
			    $SUDO cp -ar "$PWD"$PROFILE_DIR/$PROFILE_SH $PROFILE_DIR
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 15"
				exit 15
			    fi
			    $SUDO chmod 644 $PROFILE_DIR/$PROFILE_SH
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 16"
				exit 16
			    fi
			    $SUDO chown root:root $PROFILE_DIR/$PROFILE_SH
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 17"
				exit 17
			    fi

			    echo $ECHOOP "[$COLOR_GREEN OK $COLOR_OFF] api."
			    echo ""
			    ;;
			*)
			    echo $ECHOOP "TeliCamSDK api did not setup."
			    echo ""
			    break
			    ;;
		    esac
		else 

		    $SUDO cp -ar "$TELI_DIRNAME"$OPT_TELICAMSDK /opt/
		    if [ $? != 0 ] ; then
			echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 20"
			exit 20
		    fi		
		    
		    $SUDO cp -ar "$PWD"/documents $OPT_TELICAMSDK
		    if [ $? != 0 ] ; then
			echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 21"
			exit 21
		    fi

		    $SUDO chmod 775 $BIN_TELICAMSDK/TeliCamLinuxViewer $BIN_TELICAMSDK/execute_TeliCamLinuxViewer.sh
		    if [ $? != 0 ] ; then
		       echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 22"
		       exit 22
		    fi

		    $SUDO chown -R root:root $OPT_TELICAMSDK
		    if [ $? != 0 ] ; then
			echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 23"
			exit 23
		    fi
		    
		    if [ "$SUDO" ] ; then
			$SUDO mkdir -p $ROOT_TELICAMSDK_GENICAM
			if [ $? != 0 ] ; then
			    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 24"
			    exit 24
			fi
		    fi

		    $SUDO cp -ar "$PWD"$SO_PATH_DIR/$SO_PATH_FILE $SO_PATH_DIR
		    if [ $? != 0 ] ; then
			echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 25"
			exit 25
		    fi
		    $SUDO chmod 644 $SO_PATH_DIR/$SO_PATH_FILE
		    if [ $? != 0 ] ; then
			echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 26"
			exit 26
		    fi
		    $SUDO chown root:root $SO_PATH_DIR/$SO_PATH_FILE
		    if [ $? != 0 ] ; then
			echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 27"
			exit 27
		    fi
		    $SUDO ldconfig
		    if [ $? != 0 ] ; then
			echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 28"
			exit 28
		    fi

		    $SUDO cp -ar "$PWD"$PROFILE_DIR/$PROFILE_SH $PROFILE_DIR
		    if [ $? != 0 ] ; then
			echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 29"
			exit 29
		    fi
		    $SUDO chmod 644 $PROFILE_DIR/$PROFILE_SH
		    if [ $? != 0 ] ; then
			echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 30"
			exit 30
		    fi
		    $SUDO chown root:root $PROFILE_DIR/$PROFILE_SH
		    if [ $? != 0 ] ; then
			echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 31"
			exit 31
		    fi

		    echo $ECHOOP "[$COLOR_GREEN OK $COLOR_OFF] api."
		    echo ""
		fi
		
		echo "Setting up TeliCamSDK rules."
		if [ -e "$RULE_DIR/$TELI_RULE_FILE" ] ; then
		    read -p "TeliCamSDK rules exists. Do you want to setup the TeliCamSDK rules?[y/n]:" RulesReinstallYesNo
		    case $RulesReinstallYesNo in
			[Yy]*)
			    $SUDO rm -rf $RULE_DIR/$TELI_RULE_FILE
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 40"
				exit 40
			    fi
			    
			    $SUDO chmod 644 -R "$TELI_DIRNAME"$OPT_TELICAMSDK/$TELI_RULE_FILE
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 41"
				exit 41
			    fi
			    
			    $SUDO cp -ar "$TELI_DIRNAME"$OPT_TELICAMSDK/$TELI_RULE_FILE $RULE_DIR/$TELI_RULE_FILE
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 42"
				exit 42
			    fi
			    
			    $SUDO chown -R root:root $RULE_DIR/$TELI_RULE_FILE
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 43"
				exit 43
			    fi
			    
			    $SUDO udevadm control --reload-rules
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 44"
				exit 44
			    fi
			    
			    echo $ECHOOP "[$COLOR_GREEN OK $COLOR_OFF] rules."
			    echo ""
			    ;;
			*)
			    echo "TeliCamSDK rules did not setup."
			    echo ""
			    break
			    ;;
		    esac
		else
		    $SUDO chmod 644 -R "$TELI_DIRNAME"$OPT_TELICAMSDK/$TELI_RULE_FILE
		    if [ $? != 0 ] ; then
			echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 50"
			exit 50
		    fi
		    
		    $SUDO cp -ar "$TELI_DIRNAME"$OPT_TELICAMSDK/$TELI_RULE_FILE $RULE_DIR/$TELI_RULE_FILE
		    if [ $? != 0 ] ; then
			echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 51"
			exit 51
		    fi

		    $SUDO chown -R root:root $RULE_DIR/$TELI_RULE_FILE
		    if [ $? != 0 ] ; then
			echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 52"
			exit 52
		    fi
		    
		    $SUDO udevadm control --reload-rules
		    if [ $? != 0 ] ; then
			echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 53"
			exit 53
		    fi

		    echo $ECHOOP "[$COLOR_GREEN OK $COLOR_OFF] rules."
		    echo ""
		fi

		echo "Setting up TeliCamSDK samples."
		if [ -e "$HOME_DIR_TELICAMSDK" ] ; then
		    if [ -e "$HOME_DIR_TELICAMSDK_GENICAM" ] ; then
			rm -rf "$HOME_DIR_TELICAMSDK_GENICAM"
			mkdir -p "$HOME_DIR_TELICAMSDK_GENICAM"/cache
		    else 
			mkdir -p "$HOME_DIR_TELICAMSDK_GENICAM"/cache
		    fi

		    read -p "TeliCamSDK samples exists. Do you want to setup the TeliCamSDK samples?[y/n]:" SamplesReinstallYesNo
		    case $SamplesReinstallYesNo in
			[Yy]*)
			    rm -rf "$HOME_DIR_TELICAMSDK"
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 60"
				exit 60
			    fi
			    
			    cp -apr "$TELI_DIRNAME"$HOME_TELICAMSDK "$HOME"
			    if [ $? != 0 ] ; then
				echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 61"
				exit 61
			    fi
			    
			    echo $ECHOOP "[$COLOR_GREEN OK $COLOR_OFF] samples."
			    echo ""
			    ;;
			*)
			    echo "TeliCamSDK samples did not setup."
			    echo ""
			    break
			    ;;
		    esac
		else    
		    cp -apr "$TELI_DIRNAME"$HOME_TELICAMSDK "$HOME"
		    if [ $? != 0 ] ; then
			echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 70"
			exit 70
		    fi

		    echo $ECHOOP "[$COLOR_GREEN OK $COLOR_OFF] samples."
		    echo ""
		fi
		
		if [ -e "$TELICAMVIEWER_LAUNCHER" ] ; then
		    $SUDO rm $TELICAMVIEWER_LAUNCHER
		    if [ $? != 0 ] ; then
			echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 71"
			exit 71
		    fi
		fi
		
		$SUDO sh -c "echo -n > $TELICAMVIEWER_LAUNCHER"
		if [ $? != 0 ] ; then
		    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 80"
		    exit 80
		fi
		
		$SUDO sh -c "echo '[Desktop Entry]' >>$TELICAMVIEWER_LAUNCHER"
		if [ $? != 0 ] ; then
		    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 81"
		    exit 81
		fi
		
		$SUDO sh -c "echo 'Encoding=UTF-8' >>$TELICAMVIEWER_LAUNCHER"
		if [ $? != 0 ] ; then
		    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 82"
		    exit 82
		fi
		
		$SUDO sh -c "echo 'Version=0.0' >>$TELICAMVIEWER_LAUNCHER"
		if [ $? != 0 ] ; then
		    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 83"
		    exit 83
		fi
		
		$SUDO sh -c "echo 'Type=Application' >>$TELICAMVIEWER_LAUNCHER"
		if [ $? != 0 ] ; then
		    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 84"
		    exit 84
		fi
		
		$SUDO sh -c "echo 'Name=TeliCamLinuxViewer' >>$TELICAMVIEWER_LAUNCHER"
		if [ $? != 0 ] ; then
		    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 85"
		    exit 85
		fi
		
		$SUDO sh -c "echo 'Terminal=false' >>$TELICAMVIEWER_LAUNCHER"
		if [ $? != 0 ] ; then
		    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 86"
		    exit 86
		fi
		
		$SUDO sh -c "echo 'Path=/opt/TeliCamSDK/bin' >>$TELICAMVIEWER_LAUNCHER"
		if [ $? != 0 ] ; then
		    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 87"
		    exit 87
		fi
		
		$SUDO sh -c "echo 'Exec=/opt/TeliCamSDK/bin/execute_TeliCamLinuxViewer.sh' >>$TELICAMVIEWER_LAUNCHER"
		if [ $? != 0 ] ; then
		    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 88"
		    exit 88
		fi

		$SUDO sh -c "echo 'Icon=/opt/TeliCamSDK/bin/.Viewer_Main_icon_2.ico' >>$TELICAMVIEWER_LAUNCHER"
		if [ $? != 0 ] ; then
		    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 89"
		    exit 89
		fi

		$SUDO sh -c "echo 'StartupNotify=false' >>$TELICAMVIEWER_LAUNCHER"
		if [ $? != 0 ] ; then
		    echo $ECHOOP "[ \033[31;1mERROR $COLOR_OFF] exit code 90"
		    exit 90
		fi

		$SUDO sh -c "echo 'StartupWMClass=TeliCamLinuxViewer' >>$TELICAMVIEWER_LAUNCHER"
		if [ $? != 0 ] ; then
		    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 91"
		    exit 91
		fi
		
		$SUDO sh -c "echo 'X-UnityGenerated=true' >>$TELICAMVIEWER_LAUNCHER"
		if [ $? != 0 ] ; then
		    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 92"
		    exit 92
		fi

		$SUDO sh -c "echo 'Categories=Development;' >>$TELICAMVIEWER_LAUNCHER"
		if [ $? != 0 ] ; then
		    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 93"
		    exit 93
		fi

		$SUDO chown root:root $TELICAMVIEWER_LAUNCHER
		if [ $? != 0 ] ; then
		    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 94"
		    exit 94
		fi

		$SUDO chmod 644 $TELICAMVIEWER_LAUNCHER
		if [ $? != 0 ] ; then
		    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 95"
		    exit 95
		fi

		echo "Setup has completed successfully."
		echo ""

		read -p "Launch TeliCamLinuxViewer?[y/n]:" LaunchTeliCamViewerYesNo
		case $LaunchTeliCamViewerYesNo in
		    [Yy]*)
			sh $OPT_TELICAMSDK/bin/execute_TeliCamLinuxViewer.sh &
			if [ $? != 0 ] ; then
			    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 100"
			    exit 100
			fi
			;;
		    *)
			break
			;;
		esac
		;;
	    [Nn]*) echo "Exit Setup."
		;;
	    *) echo "Exit Setup."
		echo "Please Select.('y' or 'n')"
		;;
	esac
	;;
    [Nn]*) echo "Exit Setup."
	;;
    *) echo "Exit Setup."
	echo "Please Select.('y' or 'n')"
	;;
esac

if [ "$SUDO" ] ; then
    $SUDO -k
fi
