#ifndef MULTICAMERA_H
#define MULTICAMERA_H

#include <QMainWindow>
#include <QMainWindow>
#include <qtextedit.h>
#include <QDoubleSpinBox>
#include <qgroupbox.h>
#include <qlabel.h>

#include "TeliCamApi.h"
#include "dispthread.h"

using namespace Teli;

#define MAX_CAMERA      (4)


namespace Ui {
class MultiCamera;
}

class MultiCamera : public QMainWindow
{
    Q_OBJECT

public:
    explicit MultiCamera(QWidget *parent = 0);
    ~MultiCamera();

private slots:
    void on_pushButtonUpdateCameraList_clicked();

    void on_pushButtonStartAcquisition_clicked();

    void on_pushButtonStopAcquisition_clicked();

    void on_doubleSpinBoxGain1_valueChanged(double arg1);

    void on_doubleSpinBoxFrameRate1_valueChanged(double arg1);

    void on_doubleSpinBoxGain2_valueChanged(double arg1);

    void on_doubleSpinBoxGain3_valueChanged(double arg1);

    void on_doubleSpinBoxGain4_valueChanged(double arg1);

    void on_doubleSpinBoxFrameRate2_valueChanged(double arg1);

    void on_doubleSpinBoxFrameRate3_valueChanged(double arg1);

    void on_doubleSpinBoxFrameRate4_valueChanged(double arg1);

    void on_tabWidget_currentChanged(int index);

public slots:
    void disp_fps(int camNo, double fps);
    void disp_image(int camNo, QImage pimage);

private:
    Ui::MultiCamera *ui;

public:
    uint32_t				m_uiCamCount;							// Number of cameras shown in main form.
    bool8_t					m_bAcquisActive;						// Acquisition process active flag

    bool8_t					m_bFrmIndexValid[MAX_CAMERA];			// Flag that m_uiFrmIndex has valid value.
    CAM_STRM_HANDLE			m_hStrm[MAX_CAMERA];					// Handle of stream channel assigned by TeliCAMApi on opening stream channel.
    CAM_PIXEL_FORMAT		m_uiCamPixFormat[MAX_CAMERA];			// PixelFormat of camera image.
    void					*m_pvImgBGRA[MAX_CAMERA];				// Current BGRA image converted from m_pucLatestImg.
    uint32_t				m_uiCamImgWidth[MAX_CAMERA];			// Width of camera image
    uint32_t				m_uiCamImgHeight[MAX_CAMERA];			// Height of camera image
    uint32_t				m_uiFrmCount[MAX_CAMERA];				// Frame counter for calculating "Frame per second".


private:
    bool8_t                 m_bIsOpen;
    uint32_t                m_uiSelectedTabIndex;

    QGroupBox*              pArrayGroupBox[MAX_CAMERA];
    QDoubleSpinBox*         pArrayDoubleSpinBoxGain[MAX_CAMERA];
    QDoubleSpinBox*         pArrayDoubleSpinBoxFrameRate[MAX_CAMERA];
    QLabel*                 pArrayLabelInfo[MAX_CAMERA];
    QLabel*                 pArrayLabelFps[MAX_CAMERA];
    QLabel*                 pArrayLabelImage[MAX_CAMERA];

    // Display Thread
    DispThread*             m_hImgUpdThrd;                          // Thread handle of image updating thread.

    CAM_HANDLE				m_hCam[MAX_CAMERA];						// Handle of camera assigned by TeliCAMApi on opening camera.

    uint32_t				m_uiFrmIndex[MAX_CAMERA];				// Frame index.

private:
    void SetError(QString text);

    bool8_t Initialize();
    bool8_t SetFreeRunningMode();
    bool8_t GetCameraParameter();
    bool8_t SetPixelFormatInfo(uint32_t uiIndex);
    bool8_t OpenStream();
    bool8_t CloseStream();
    bool8_t StartAcquisition();
    bool8_t StopAcquisition();
    bool8_t TerminateImgThrd();

    void Terminate();

    void CloseAllCameras();
    void UpdateCameraInfo();
    void UpdateGain();
    void UpdateFrameRate();

    void SetGain(uint32_t camNo, double value);
    void SetFramerate(uint32_t camNo, double value);
    void DispInfo(uint32_t camNo, QString message);

public:
    static void CallBackImageAcquired(CAM_HANDLE hCam,
                                        CAM_STRM_HANDLE hStrm,
                                        CAM_IMAGE_INFO *psImageInfo,
                                        uint32_t uiBufferIndex,
                                        void *pvContext);				// Callback for handling received image.

    static void CallBackErrorImage(CAM_HANDLE hCam,
                                        CAM_STRM_HANDLE hStrm,
                                        CAM_API_STATUS uiErrorStatus,
                                        uint32_t uiBufferIndex,
                                        void *pvContext);				// Callback for handling receive error.

    static void CallbackBufferBusy(CAM_HANDLE hCam,
                                        CAM_STRM_HANDLE hRcvStrm,
                                        uint32_t uiBufferIndex,
                                        void* pvContext);

};

#endif // MULTICAMERA_H
