///////////////////////////////////////////////////////////////////////////////
// file    : TeliCamUtl.h
// brief   : Teli camera utility dll.
// version : 1.0.0.1
// author  : (C)2014 TOSHIBA TELI CORPORATION
// version : 1.0.0.1
// date    : 2015/04/27
//   Renamed the first argument name of  ConvByr??8ToBGRACP().
//
// version : 1.0.0.0
// date    : 2014/09/02
// TeliCamUtl.h

#ifndef _TELICAMUTL_H_
#define _TELICAMUTL_H_


#include "TeliCamApi.h"

namespace Teli
{

	//
	#ifdef __cplusplus
		#ifndef EXTERN_C
			#define EXTERN_C extern "C"
		#endif
	#else
		#ifndef EXTERN_C
			#define EXTERN_C
		#endif
	#endif

	#if defined (_WIN32)
		#ifdef CAMUTL_EXPORTS
			#define TELICAMUTL_API
		#else
			#define TELICAMUTL_API __declspec(dllimport)
		#endif

		#define IMGAPI EXTERN_C 	TELICAMUTL_API CAM_API_STATUS _stdcall
		#define IMGAPIBYTE EXTERN_C TELICAMUTL_API uint8_t _stdcall
		#define IMGAPIBOOL EXTERN_C TELICAMUTL_API bool8_t _stdcall
	#else
		#define TELICAMUTL_API

		#define IMGAPI EXTERN_C 	TELICAMUTL_API CAM_API_STATUS
		#define IMGAPIBYTE EXTERN_C TELICAMUTL_API uint8_t
		#define IMGAPIBOOL EXTERN_C TELICAMUTL_API bool8_t
	#endif

	typedef enum _DST_FORMAT
	{
		DST_FMT_BGRA32		=   0,		// BGRA 32bit format (  0)
		DST_FMT_BGR24		=   1,		// BGR  24bit format (  1)
	} DST_FORMAT;

	// Initialize.
	IMGAPI PrepareLUT();																	// Prepare Look up table for Yuv -> BGR conversion. Call this function once before using YUV type ARGB converter.

	//Utility
	IMGAPIBYTE BitPerPixel(CAM_PIXEL_FORMAT  uiPixelFormat);										// Get bit per pixel data.
	IMGAPIBYTE DataDepth(CAM_PIXEL_FORMAT  uiPixelFormat);										// Get data depth of color component or pixel data.
	IMGAPIBOOL IsMonochromic(CAM_PIXEL_FORMAT  uiPixelFormat);									// If monochrome format, return true. False will be returned when bayer type PixelFormat is speccified.
	IMGAPIBOOL IsPixelBayer(CAM_PIXEL_FORMAT  uiPixelFormat);										// If bayer type format, return true.

	IMGAPI SaveBmpARGB(void *pvTgt,	uint32_t uiWidth, uint32_t uiHeight, const char *pszPath);	// Save image as ARGB 32bit bitmap.
	IMGAPI SaveBmpRGB(void *pvTgt,uint32_t uiWidth, uint32_t uiHeight, const char *pszPath);	// Save image as  RGB 24bit bitmap.
	IMGAPI SaveBmpMono(void *pvTgt,	uint32_t uiWidth, uint32_t uiHeight, const char *pszPath);	// Save image as monochrome 8nit bitmap.

	IMGAPI SaveRWH(void *pvTgt, CAM_PIXEL_FORMAT uiPixelFormat, uint32_t uiWidth, uint32_t uiHeight, const char *pszPath);// Save image as original format "Raw data With Header" data.
	IMGAPI LoadRWH(const char *pszPath, void **ppTgt, CAM_PIXEL_FORMAT *puiPixelFormat = NULL, uint32_t *puiWidth = NULL, uint32_t *puiHeight = NULL, uint32_t *puiStride = NULL);// Load original format "Raw data With Header" data as image.

	IMGAPI ReverseImg(void *pvDst, void *pvSrc, CAM_PIXEL_FORMAT uiPixelFormat, uint32_t uiWidth, uint32_t uiHeight,  bool bRevX, bool bRevY);// Reverse Image.

	// Image Converter
	IMGAPI ConvImage(DST_FORMAT eDstFormat, CAM_PIXEL_FORMAT uiSrcPixelFormat, bool bBayerConversion, void *pvDst, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);


	// BGRA Converter
	//  for monochrome source data
	IMGAPI ConvMono8ToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Mono8 -> BGRA
	IMGAPI ConvMono10ToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Mono10 -> BGRA
	IMGAPI ConvMono12ToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Mono12 -> BGRA
	IMGAPI ConvMono16ToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Mono16 -> BGRA

	//  for bayer filter sensor raw source data (pixel data depth : 8bit)
	IMGAPI ConvByrGR8ToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Bayer GR8 -> BGRA
	IMGAPI ConvByrRG8ToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Bayer RG8 -> BGRA
	IMGAPI ConvByrGB8ToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Bayer GB8 -> BGRA
	IMGAPI ConvByrBG8ToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Bayer BG8 -> BGRA

	//  ACPI for bayer filter sensor raw source data (pixel data depth : 8bit)
	IMGAPI ConvByrGR8ToBGRAACPI(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);	// Bayer GR8 -> BGRA
	IMGAPI ConvByrRG8ToBGRAACPI(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);	// Bayer RG8 -> BGRA
	IMGAPI ConvByrGB8ToBGRAACPI(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);	// Bayer GB8 -> BGRA
	IMGAPI ConvByrBG8ToBGRAACPI(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);	// Bayer BG8 -> BGRA

	//  for bayer filter sensor raw source data (pixel data depth : 10bit)
	IMGAPI ConvByrGR10ToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);	// Bayer GR10 -> BGRA
	IMGAPI ConvByrRG10ToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);	// Bayer RG10 -> BGRA
	IMGAPI ConvByrGB10ToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);	// Bayer GB10 -> BGRA
	IMGAPI ConvByrBG10ToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);	// Bayer BG10 -> BGRA

	//  for bayer filter sensor raw source data (pixel data depth : 12bit)
	IMGAPI ConvByrGR12ToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);	// Bayer GR12 -> BGRA
	IMGAPI ConvByrRG12ToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);	// Bayer RG12 -> BGRA
	IMGAPI ConvByrGB12ToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);	// Bayer GB12 -> BGRA
	IMGAPI ConvByrBG12ToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);	// Bayer BG12 -> BGRA

	//  for RGB or BGR format source data (pixel data depth : 8bit)
	IMGAPI ConvRGB8PToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// RGB8 -> BGRA
	IMGAPI ConvBGR8PToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// BGR8 -> BGRA

	//  for RGB or BGR format source data (pixel data depth : 10bit)
	IMGAPI ConvRGB10PToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// RGB10 -> BGRA
	IMGAPI ConvBGR10PToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// BGR10 -> BGRA

	//  for RGB or BGR format source data (pixel data depth : 12bit)
	IMGAPI ConvRGB12PToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// RGB12 -> BGRA
	IMGAPI ConvBGR12PToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// BGB12 -> BGRA

	// for YUV format
	IMGAPI ConvYUV411PToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);	// YUV411 -> BGRA
	IMGAPI ConvYUV422PToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);	// YUV422 -> BGRA
	IMGAPI ConvYUV444PToBGRA(void *pvDstBGRA, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);	// YUV444 -> BGRA


	// RGB24bit Converter
	//  for monochrome source data
	IMGAPI ConvMono8ToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeightt);		// Mono8 -> BGR
	IMGAPI ConvMono10ToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Mono10 -> BGR
	IMGAPI ConvMono12ToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Mono12 -> BGR
	IMGAPI ConvMono16ToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Mono16 -> BGR

	//  for bayer filter sensor raw source data (pixel data depth : 8bit)
	IMGAPI ConvByrGR8ToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Bayer GR8 -> BGR
	IMGAPI ConvByrRG8ToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Bayer RG8 -> BGR
	IMGAPI ConvByrGB8ToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Bayer GB8 -> BGR
	IMGAPI ConvByrBG8ToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Bayer BG8 -> BGR

	//  ACPI for bayer filter sensor raw source data (pixel data depth : 8bit)
	IMGAPI ConvByrGR8ToBGRACPI(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);	// Bayer GR8 -> BGR
	IMGAPI ConvByrRG8ToBGRACPI(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);	// Bayer RG8 -> BGR
	IMGAPI ConvByrGB8ToBGRACPI(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);	// Bayer GB8 -> BGR
	IMGAPI ConvByrBG8ToBGRACPI(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);	// Bayer BG8 -> BGR

	//  for bayer filter sensor raw source data (pixel data depth : 10bit)
	IMGAPI ConvByrGR10ToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Bayer GR10 -> BGR
	IMGAPI ConvByrRG10ToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Bayer RG10 -> BGR
	IMGAPI ConvByrGB10ToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Bayer GB10 -> BGR
	IMGAPI ConvByrBG10ToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Bayer BG10 -> BGR

	//  for bayer filter sensor raw source data (pixel data depth : 12bit)
	IMGAPI ConvByrGR12ToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Bayer GR12 -> BGR
	IMGAPI ConvByrRG12ToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Bayer RG12 -> BGR
	IMGAPI ConvByrGB12ToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Bayer GB12 -> BGR
	IMGAPI ConvByrBG12ToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// Bayer BG12 -> BGR

	//  for RGB or BGR format source data (pixel data depth : 8bit)
	IMGAPI ConvRGB8PToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// RGB8 -> BGR
	IMGAPI ConvBGR8PToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// BGR8 -> BGR

	//  for RGB or BGR format source data (pixel data depth : 10bit)
	IMGAPI ConvRGB10PToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// RGB10 -> BGR
	IMGAPI ConvBGR10PToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// BGR10 -> BGR

	//  for RGB or BGR format source data (pixel data depth : 12bit)
	IMGAPI ConvRGB12PToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// RGB12 -> BGR
	IMGAPI ConvBGR12PToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// BGB12 -> BGR

	// for YUV format
	IMGAPI ConvYUV411PToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// YUV411 -> BGR
	IMGAPI ConvYUV422PToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// YUV422 -> BGR
	IMGAPI ConvYUV444PToBGR(void *pvDstBGR, void *pvSrc, uint32_t uiWidth, uint32_t uiHeight);		// YUV444 -> BGR



	IMGAPI SetBayerConvMode(uint32_t uiBCMode = 0);				// Bayer Conversion mode 1: ACPI, The Other : Bi-Linear
}

#endif // _TELICAMUTL_H_
