#!/bin/sh

cd `dirname $0`

COLOR_RED="\033[1;31m"
COLOR_GREEN="\033[1;32m"
COLOR_OFF="\033[m"

OPT_TELICAMSDK=/opt/TeliCamSDK
HOME_DIR_TELICAMSDK=$HOME/TeliCamSDK
TELI_RULE_FILE=80-telicamsdk.rules
RULE_DIR=/etc/udev/rules.d
TELICAMVIEWER_LAUNCHER=/usr/share/applications/telicamlnxviewer.desktop
ROOT_TELICAMSDK=/root/TeliCamSDK

ARCH=`getconf LONG_BIT`
case $ARCH in
    64)
	SO_PATH_FILE=telicamsdk_x64.conf
	;;
    32)
	SO_PATH_FILE=telicamsdk_x86.conf
	;;
    *)
	;;
esac

echo ""

read -p "Do you want to remove TeliCamSDK?[y/n]:" RemoveYesNo

case $RemoveYesNo in
    [Yy]*)
	SUDO=""
	if [ `id -u` != "0" ] ; then
	    SUDO="sudo"
	fi

	ECHOOP=""
	if [ -e /etc/redhat-release ] ; then
	    GETDISTRIB=`cat /etc/redhat-release | awk 'NR == 1 {print $1}'`
	else
	    GETDISTRIB=`cat /etc/issue | awk 'NR == 1 {print $1}'`
	fi

	case $GETDISTRIB in
	    Ubuntu)
		;;
	    Debian)
		;;
	    Fedora)
		ECHOOP="-e"
		;;
	    CentOS)
		ECHOOP="-e"
		;;
	    *)
		;;
	esac

	echo ""

	echo "Removing TeliCamSDK api."
	if [ -e "$OPT_TELICAMSDK" ] ; then
	    
	    if [ -e /etc/ld.so.conf.d/"$SO_PATH_FILE" ] ; then
		$SUDO rm -rf /etc/ld.so.conf.d/$SO_PATH_FILE
		if [ $? != 0 ] ; then
		    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 11"
		    exit 1
		fi
		$SUDO ldconfig
	    fi

	    $SUDO rm -rf $OPT_TELICAMSDK
	    if [ $? != 0 ] ; then
		echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 1"
		exit 2
	    fi

	    if [ `id -u` != "0" ] ; then
		$SUDO rm -rf $ROOT_TELICAMSDK
		if [ $? != 0 ] ; then
		    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 10"
		    exit 10
		fi
	    fi

	    if [ -e /etc/profile.d/telicamsdk.sh ] ; then
		$SUDO rm -rf /etc/profile.d/telicamsdk.sh
		if [ $? != 0 ] ; then
		    echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 12"
		    exit 11
		fi
	    fi

	    echo $ECHOOP "[$COLOR_GREEN OK $COLOR_OFF] api."
	else 
	    echo "Not exist."
	fi

	echo ""

	echo "Removing TeliCamSDK rules."
	if [ -e "$RULE_DIR/$TELI_RULE_FILE" ] ; then 
	    $SUDO rm -rf $RULE_DIR/$TELI_RULE_FILE
	    if [ $? != 0 ] ; then
		echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 20"
		exit 20
	    fi
	    echo $ECHOOP "[$COLOR_GREEN OK $COLOR_OFF] rules."
	else 
	    echo "Not exist."
	fi

	echo ""

	echo "Removing TeliCamSDK samples."
	if [ -e "$HOME_DIR_TELICAMSDK" ] ; then
	    $SUDO rm -rf "$HOME_DIR_TELICAMSDK"
	    if [ $? != 0 ] ; then
		echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 30"
		exit 30
	    fi

	    echo $ECHOOP "[$COLOR_GREEN OK $COLOR_OFF] samples."
	else 
	    echo "Not exist."
	fi

	echo ""

	if [ -e "$TELICAMVIEWER_LAUNCHER" ] ; then 
	    $SUDO rm -rf $TELICAMVIEWER_LAUNCHER
	    if [ $? != 0 ] ; then
		echo $ECHOOP "[$COLOR_RED ERROR $COLOR_OFF] exit code 40"
		exit 40
	    fi
	fi

	echo "TeliCamSDK was successfully removed from your computer."
	echo ""
	;;
    [Nn]*) 
	echo "Exit."
	;;
    *) 
	echo "Exit."
	echo "Please Select.('y' or 'n')"
	;;
esac

if [ "$SUDO" ] ; then
    $SUDO -k
fi
