#include <QElapsedTimer>
#include <QMessageBox>

#include "dispthread.h"
#include "multicamera.h"
#include "TeliCamUtl.h"

DispThread::DispThread(QObject *parent) :
    QThread(parent)
{
    this->pParent = parent;
}

void DispThread::run()
{
    MultiCamera* pParent = (MultiCamera*)this->pParent;
    uint32_t		uiInterval = 0;
    uint32_t		uiIndexRBuf;
    CAM_IMAGE_INFO	sImgInfo;
    CAM_API_STATUS	status;
    QElapsedTimer   et;
    double          fps;
    bool            bDispFpsFirst = true;

    et.start();

    while(!this->m_bTerminate)
    {
        usleep(50000);	// Image update cycle 50msec in this sample. (20 Image / sec)

        if (pParent->m_bAcquisActive)
        {
            for(uint32_t idx = 0; idx < pParent->m_uiCamCount; ++idx)
            {
                if(pParent->m_bFrmIndexValid[idx] == true)
                {	//Image received.
                    // Get current ring buffer.
                    status = Strm_GetCurrentBufferIndex(pParent->m_hStrm[idx], &uiIndexRBuf);
                    if(CAM_API_STS_SUCCESS == status)
                    {
                        // Get image pointer in Ring Buffer. Image data is locked in Strm_LockApiRingBufferPointer() method.
                        status = Strm_LockBuffer(pParent->m_hStrm[idx], uiIndexRBuf, &sImgInfo);
                        if((status == CAM_API_STS_SUCCESS) && (sImgInfo.uiStatus == CAM_API_STS_SUCCESS))
                        {
                            // Convert received image data to BGRA pixel format (not Bitmap).
                            status = ConvImage(DST_FMT_BGRA32,
                                                pParent->m_uiCamPixFormat[idx],
                                                true,
                                                pParent->m_pvImgBGRA[idx],
                                                sImgInfo.pvBuf,
                                                pParent->m_uiCamImgWidth[idx],
                                                pParent->m_uiCamImgHeight[idx]);
                        }
                        // Unlock current image
                        Strm_UnlockBuffer(pParent->m_hStrm[idx], uiIndexRBuf);

                        // Update image
                        QImage image;
                        image = QImage((unsigned char*)pParent->m_pvImgBGRA[idx], pParent->m_uiCamImgWidth[idx], pParent->m_uiCamImgHeight[idx], QImage::Format_ARGB32);

                        emit DispImage(idx, image);

                    }
                }
            }

            // Check interval
            uiInterval = et.elapsed();
            if (uiInterval > 2000)    // 2000ms
            {
                if (bDispFpsFirst)
                {
                    bDispFpsFirst = false;
                    for (uint32_t idx=0; idx<pParent->m_uiCamCount; idx++)
                    {
                        pParent->m_uiFrmCount[idx] = 0;
                    }
                } else
                {	// Update FPS
                    for (uint32_t idx=0; idx<pParent->m_uiCamCount; idx++)
                    {
                        fps = (double)pParent->m_uiFrmCount[idx] * 1000 / (double)uiInterval;
                        pParent->m_uiFrmCount[idx] = 0;

                        emit DispFps(idx, fps);
                    }
                 }
                et.restart();
            }
        }
    }
}




